@@ -1,8 +1,9 @@
 from datetime import datetime
 
 from django.db import connection
-from django.db.models import OuterRef, Subquery
+from django.db.models import OuterRef, Subquery, Q
 from temba.contacts.models import Contact
+from temba.channels.models import Channel
 from temba.msgs.models import Msg
 from temba.orgs.models import Org
 
@@ -49,9 +50,13 @@ class ActiveContactsQuery:
                 msg__sent_on=Subquery(msg.values("sent_on")[:1]),
                 msg__direction=Subquery(msg.values("direction")[:1]),
                 channel__uuid=Subquery(msg.values("channel__uuid")[:1]),
+                channel__id=Subquery(msg.values("channel_id")[:1]),
                 channel__name=Subquery(msg.values("channel__name")[:1]),
             )
-            .filter(org=org, msg__uuid__isnull=False,)
+            .filter(
+                org=org,
+                msg__uuid__isnull=False,
+            )
             .values(
                 "uuid",
                 "name",
@@ -60,6 +65,33 @@ class ActiveContactsQuery:
                 "msg__sent_on",
                 "msg__direction",
                 "channel__uuid",
-                "channel__name"
+                "channel__id",
+                "channel__name",
             )
         )
+
+
+class MessageDetailQuery:
+    @classmethod
+    def incoming_message(cls, org_uuid: str, contact_uuid: str, before: datetime, after: datetime):
+        org = Org.objects.get(uuid=org_uuid)
+        contact = Contact.objects.get(uuid=contact_uuid)
+
+        msg = (
+            Msg.objects.filter(created_on__lte=before, created_on__gte=after, org=org, contact=contact)
+            .filter(Q(direction="I") | Q(direction="O"))
+            .exclude(status="F")
+            .order_by("-created_on")
+            .values("uuid", "text", "created_on", "direction", "channel")
+            .last()
+        )
+
+        if not msg:
+            return None
+
+        channel = Channel.objects.get(id=msg["channel"])
+
+        msg["channel_id"] = channel.id
+        msg["channel_type"] = channel.channel_type
+
+        return msg
